from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .PredefinedFieldState import PredefinedFieldState
from ..UtilityAndView.abaqusConstants import Boolean, NONE, OFF, SymbolicConstant, UNSET


@abaqus_class_doc
class TemperatureState(PredefinedFieldState):
    """The TemperatureState object stores the propagating data of a temperature in a step. One
    instance of this object is created internally by the Temperature object for each step.
    The TemperatureState object has no constructor or methods.
    The TemperatureState object is derived from the PredefinedFieldState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].predefinedFieldStates[name]
    """

    #: A SymbolicConstant specifying the propagation state of the **fileName** member. Possible
    #: values are UNSET, SET, and UNCHANGED.
    fileNameState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant or an Int specifying the first step from which temperature values are
    #: to be read. This argument is valid only when **distribution** = FROM_FILE or
    #: **distribution** = FROM_FILE_AND_USER_DEFINED. Possible values are FIRST_STEP, LAST_STEP,
    #: and NONE. The default value is NONE.
    beginStep: SymbolicConstant = NONE

    #: A SymbolicConstant specifying the propagation state of the **beginStep** member. Possible
    #: values are UNSET, SET, and UNCHANGED.
    beginStepState: Optional[SymbolicConstant] = None

    #: None or an Int specifying the first increment of the step set in **beginStep** or the
    #: SymbolicConstants STEP_START or STEP_END. This argument is valid only when
    #: **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
    #: default value is None.
    beginIncrement: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **beginIncrement** member.
    #: Possible values are UNSET, SET, and UNCHANGED.
    beginIncrementState: Optional[SymbolicConstant] = None

    #: None or an Int specifying the last step from which temperature values are to be read or
    #: the SymbolicConstants FIRST_STEP and LAST_STEP. This argument is valid only when
    #: **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
    #: default value is None.
    endStep: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **endStep** member. Possible
    #: values are UNSET, SET, and UNCHANGED.
    endStepState: Optional[SymbolicConstant] = None

    #: None or an Int specifying the last increment of the step set in **endStep** or the
    #: SymbolicConstants STEP_START and STEP_END. This argument is valid only when
    #: **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
    #: default value is None.
    endIncrement: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **endIncrement** member.
    #: Possible values are UNSET, SET, and UNCHANGED.
    endIncrementState: Optional[SymbolicConstant] = None

    #: A Boolean specifying that temperatures in second-order elements are to be interpolated
    #: from corner node temperatures. This argument is valid only when
    #: **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED.
    midside: Boolean = OFF

    #: A SymbolicConstant specifying the propagation state of the **midside** member. Possible
    #: values are UNSET, SET, and UNCHANGED.
    midsideState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitudeState** member.
    #: Possible values are UNSET, SET, and UNCHANGED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A String specifying the name of the file from which the temperature values are to be
    #: read when **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED.
    fileName: str = ""

    #: The SymbolicConstant UNSET or a String specifying the name of the amplitude reference.
    #: The SymbolicConstant UNSET should be used if the predefined field has no amplitude
    #: reference. The default value is UNSET.Note:*amplitude* should be given only if it is
    #: valid for the specified step.
    amplitude: SymbolicConstant = UNSET

    #: A tuple of SymbolicConstants specifying the propagation state of each item of the
    #: **magnitudes** member. Possible values are UNSET, SET, and UNCHANGED.
    magnitudesState: Optional[SymbolicConstant] = None

    #: A tuple of Floats specifying the temperature values when **distributionType** = UNIFORM or
    #: **distributionType** = FIELD. The value of the **magnitudes** argument is a function of the
    #: **crossSectionDistribution** argument, as shown in the following list:
    #: - If **crossSectionDistribution** = CONSTANT_THROUGH_THICKNESS then **magnitudes** is a Double
    #: specifying the temperature.
    #: - If **crossSectionDistribution** = GRADIENTS_THROUGH_SHELL_CS then **magnitudes** is a
    #: sequence of Doubles specifying the mean value and the gradient in the thickness
    #: direction.
    #: - If **crossSectionDistribution** = GRADIENTS_THROUGH_BEAM_CS then **magnitudes** is a
    #: sequence of Doubles specifying the mean value, the gradient in the N1 direction, and the
    #: gradient in the N2 direction.
    #: - If **crossSectionDistribution** = POINTS_THROUGH_SECTION then **magnitudes** is a sequence
    #: of Doubles specifying the temperature at each point.
    magnitudes: tuple = ()

    #: A SymbolicConstant specifying the propagation state of the PredefinedFieldState object.
    #: Possible values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - DEACTIVATED_TO_INITIAL
    #: - NO_LONGER_ACTIVE
    #: - RESET_TO_INITIAL
    #: - TO_BE_COMPUTED
    #: - PROPAGATED_FROM_COMPUTED
    #: - BUILT_INTO_BASE_STATE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: 
    #: This member exists in all PredefinedFieldState objects, but different predefined fields
    #: use different subsets of the entire list of possible values depending on propagation
    #: rules.
    status: Optional[SymbolicConstant] = None



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConnectorOptions import ConnectorOptions
from .ConnectorPotentialArray import ConnectorPotentialArray
from .TangentialBehavior import TangentialBehavior
from ..UtilityAndView.abaqusConstants import (Boolean, COEFFICIENTS, CONSTANT, FRACTION, LINEAR,
                                              OFF, ON, PENALTY, SymbolicConstant)


# Prevent circular import
class DerivedComponent:
    ...


@abaqus_class_doc
class ConnectorBehaviorOption:
    """The ConnectorBehaviorOption object is the abstract base type for other
    ConnectorBehaviorOption objects. The ConnectorBehaviorOption object has no explicit
    constructor. The members of the ConnectorBehaviorOption object are common to all objects
    derived from the ConnectorBehaviorOption.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name].behaviorOptions[i]
            import odbSection
            session.odbs[name].sections[name].behaviorOptions[i]
    """

    #: A :py:class:`~abaqus.Connector.ConnectorPotentialArray.ConnectorPotentialArray` object.
    connectorPotentials: ConnectorPotentialArray = []

    #: A :py:class:`~abaqus.Connector.ConnectorBehaviorOption.DerivedComponent` object.
    derivedComponent: DerivedComponent = DerivedComponent()

    #: A :py:class:`~abaqus.Connector.ConnectorPotentialArray.ConnectorPotentialArray` object.
    evolutionPotentials: ConnectorPotentialArray = []

    #: A :py:class:`~abaqus.Connector.ConnectorPotentialArray.ConnectorPotentialArray` object.
    initiationPotentials: ConnectorPotentialArray = []

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object.
    initiationOptions: ConnectorOptions = ConnectorOptions()

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object.
    isotropicOptions: ConnectorOptions = ConnectorOptions()

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object.
    kinematicOptions: ConnectorOptions = ConnectorOptions()

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object specifying the ConnectorOptions used to define tabular options
    #: for this ConnectorBehaviorOption.
    options: ConnectorOptions = ConnectorOptions()

    #: A :py:class:`~abaqus.Connector.TangentialBehavior.TangentialBehavior` object
    tangentialBehavior: TangentialBehavior = TangentialBehavior()

    @abaqus_method_doc
    def TangentialBehavior(
        self,
        formulation: SymbolicConstant = PENALTY,
        slipRateDependency: Boolean = OFF,
        pressureDependency: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        exponentialDecayDefinition: SymbolicConstant = COEFFICIENTS,
        shearStressLimit: Optional[float] = None,
        maximumElasticSlip: SymbolicConstant = FRACTION,
        fraction: Optional[float] = None,
        absoluteDistance: Optional[float] = None,
        table: tuple = (),
    ) -> TangentialBehavior:
        """This method creates a TangentialBehavior object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sections[name].behaviorOptions[i].TangentialBehavior
                session.odbs[name].sections[name].behaviorOptions[i].TangentialBehavior

        Parameters
        ----------
        formulation
            A SymbolicConstant specifying the friction coefficient formulation. Possible values are
            PENALTY and EXPONENTIAL_DECAY. The default value is PENALTY.
        slipRateDependency
            A Boolean specifying whether the data depend on slip rate. The default value is OFF.
        pressureDependency
            A Boolean specifying whether the data depend on contact pressure. The default value is
            OFF.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variables for the data. The default value is 0.
        exponentialDecayDefinition
            A SymbolicConstant specifying the exponential decay definition for the data. Possible
            values are COEFFICIENTS and TEST_DATA. The default value is COEFFICIENTS.
        shearStressLimit
            None or a Float specifying no upper limit or the friction coefficient shear stress
            limit. The default value is None.
        maximumElasticSlip
            A SymbolicConstant specifying the method for modifying the allowable elastic slip.
            Possible values are FRACTION and ABSOLUTE_DISTANCE. The default value is FRACTION.This
            argument applies only to Abaqus/Standard analyses.
        fraction
            A Float specifying the ratio of the allowable maximum elastic slip to a characteristic
            model dimension. The default value is 10-4.This argument applies only to Abaqus/Standard
            analyses.
        absoluteDistance
            None or a Float specifying the absolute magnitude of the allowable elastic slip. The
            default value is None.This argument applies only to Abaqus/Standard analyses.
        table
            A sequence of sequences of Floats specifying the tangential properties. Items in the
            table data are described below. The default value is an empty sequence.

        Returns
        -------
        TangentialBehavior
            A :py:class:`~abaqus.Connector.TangentialBehavior.TangentialBehavior` object.
        """
        self.tangentialBehavior = tangentialBehavior = TangentialBehavior(
            formulation,
            slipRateDependency,
            pressureDependency,
            temperatureDependency,
            dependencies,
            exponentialDecayDefinition,
            shearStressLimit,
            maximumElasticSlip,
            fraction,
            absoluteDistance,
            table,
        )
        return tangentialBehavior

    @abaqus_method_doc
    def DerivedComponent(self) -> DerivedComponent:
        """This method creates a DerivedComponent object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sections[name].behaviorOptions[i].TangentialBehavior
                session.odbs[name].sections[name].behaviorOptions[i].TangentialBehavior

        Returns
        -------
        DerivedComponent
            A :py:class:`~abaqus.Connector.ConnectorBehaviorOption.DerivedComponent` object.

        Raises
        ------
        ValueError and TextError
        """
        self.derivedComponent = derivedComponent = DerivedComponent()
        return derivedComponent

    @abaqus_method_doc
    def ConnectorOptions(
        self,
        useBehRegSettings: Boolean = ON,
        regularize: Boolean = ON,
        defaultTolerance: Boolean = ON,
        regularization: float = 0,
        defaultRateFactor: Boolean = ON,
        rateFactor: float = 0,
        interpolation: SymbolicConstant = LINEAR,
        useBehExtSettings: Boolean = ON,
        extrapolation: SymbolicConstant = CONSTANT,
    ) -> ConnectorOptions:
        """This method creates a connector options object to be used in conjunction with an
        allowable connector behavior option, derived component term, or connector section.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sections[name].behaviorOptions[i].TangentialBehavior
                session.odbs[name].sections[name].behaviorOptions[i].TangentialBehavior

        Parameters
        ----------
        useBehRegSettings
            A Boolean specifying whether or not to use the behavior-level settings for
            regularization options. This argument is applicable only for an Abaqus/Explicit
            analysis. The default value is ON.
        regularize
            A Boolean specifying whether or not the tabular data will be regularized. This argument
            is applicable only for an Abaqus/Explicit analysis and only if **useBehRegSettings** = OFF.
            The default value is ON.
        defaultTolerance
            A Boolean specifying whether or not the analysis default regularization tolerance will
            be used. This argument is applicable only for an Abaqus/Explicit analysis and only if
            **useBehRegSettings** = OFF and **regularize** = ON. The default value is ON.
        regularization
            A Float specifying the regularization increment to be used. This argument is applicable
            only for an Abaqus/Explicit analysis and only if **useBehRegSettings** = OFF,
            **regularize** = ON, and **defaultTolerance** = OFF. The default value is 0.03.
        defaultRateFactor
            A Boolean specifying whether or not the analysis default rate filter factor will be
            used. This argument is applicable only for an Abaqus/Explicit analysis that includes
            isotropic hardening with tabular definition or damage initiation with Plastic motion
            criteria. The default value is ON.
        rateFactor
            A Float specifying the rate filter factor to be used. This argument is applicable only
            for an Abaqus/Explicit analysis that includes isotropic hardening with tabular
            definition or damage initiation with Plastic motion criteria. This argument is also
            applicable only if **defaultRateFactor** = OFF. The default value is 0.9.
        interpolation
            A SymbolicConstant specifying the type of interpolation increment to be used on
            rate-dependent tabular data. This argument is applicable only for an Abaqus/Explicit
            analysis that includes isotropic hardening with tabular definition or damage initiation
            with Plastic motion criteria. Possible values are LINEAR and LOGARITHMIC. The default
            value is LINEAR.
        useBehExtSettings
            A Boolean specifying whether or not to use the behavior-level settings for extrapolation
            options. The default value is ON.
        extrapolation
            A SymbolicConstant specifying the extrapolation technique to be used. This argument is
            applicable only if **useBehExtSettings** = OFF. Possible values are CONSTANT and LINEAR. The
            default value is CONSTANT.

        Returns
        -------
        ConnectorOptions
            A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object.

        Raises
        ------
        ValueError and TextError.
        """
        self.options = connectorOptions = ConnectorOptions(
            useBehRegSettings,
            regularize,
            defaultTolerance,
            regularization,
            defaultRateFactor,
            rateFactor,
            interpolation,
            useBehExtSettings,
            extrapolation,
        )
        return connectorOptions



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class SpringDashpot:
    """The SpringDashpot object is the abstract base type for the SpringDashpotToGround and
    TwoPointSpringDashpot objects.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].engineeringFeatures.springDashpots[name]
            import assembly
            mdb.models[name].rootAssembly.engineeringFeatures.springDashpots[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A Boolean specifying whether the spring/dashpot is suppressed or not. The default value
    #: is OFF.
    suppressed: Boolean = OFF

    @abaqus_method_doc
    def resume(self):
        """This method resumes the spring/dashpot that was previously suppressed."""
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses the spring/dashpot."""
        ...



import os
import re
from typing import List


def findUserTypePath(type: str, searchDir: str):
    if not os.path.isdir(searchDir):
        return ''

    files = os.listdir(searchDir)
    for file in files:

        if os.path.isdir(os.path.join(searchDir, file)):
            foundDir = findUserTypePath(type, os.path.join(searchDir, file))
            if not foundDir == '':
                return foundDir
        elif os.path.isfile(os.path.join(searchDir, file)) and file.endswith('.py'):
            filePath = os.path.join(searchDir, file)
            with open(filePath, 'r+', encoding='utf-8') as f:
                text = f.read()
                if f'class {type}:\n' in text or f'class {type}(' in text:
                    return filePath
                f.close()
    return ''


def findModuleName(type: str, searchDir: str):
    filePath = findUserTypePath(type, searchDir)
    return os.path.relpath(filePath, '.').replace('.py', '') if filePath != '' else ''


def replaceUserType(searchDir: str):
    print(f'# Processing dir: {searchDir}')
    if not os.path.isdir(searchDir):
        return ''

    files = os.listdir(searchDir)
    for file in files:
        if os.path.isdir(os.path.join(searchDir, file)):
            replaceUserType(os.path.join(searchDir, file))
        elif os.path.isfile(os.path.join(searchDir, file)) and file.endswith('.py'):
            print(f'    # Processing file: {file}')
            filePath = os.path.join(searchDir, file)
            with open(filePath, 'r+', encoding='utf-8') as f:
                text = f.read()
                strings: List[str] = re.findall(r'An? \w+ object', text)
                for string in strings:
                    type = string.split(' ')[-2]
                    module = findModuleName(type, '.').replace('\\', '.')
                    if module == '':
                        continue
                    new_string = string.replace(type, f':py:class:`~{module}.{type}`')
                    text = text.replace(string, new_string)
                    print(f'        # Replaced {type} to :py:class:`~{module}.{type}`')
            if len(strings) > 0:
                with open(filePath, 'r+', encoding='utf-8') as f:
                    f.write(text)


# replaceUserType('abaqus')

def addAttributes(searchDir: str):
    print(f'# Processing dir: {searchDir}')
    if not os.path.isdir(searchDir):
        return ''

    files = os.listdir(searchDir)
    for file in files:
        if os.path.isdir(os.path.join(searchDir, file)):
            addAttributes(os.path.join(searchDir, file))
        elif os.path.isfile(os.path.join(searchDir, file)) and file.endswith('.md'):
            print(f'    # Processing file: {file}')
            filePath = os.path.join(searchDir, file)
            with open(filePath, 'r+', encoding='utf-8') as f:
                text = f.read()
                strings: List[str] = re.findall(
                    r'The \w+ object has members with the same names and descriptions as the arguments to the ', text)

            if len(strings) > 0:
                pyFilePath = filePath.replace('markdown', 'abaqus').replace('.md', '.py')
                if not os.path.exists(pyFilePath):
                    continue
                with open(pyFilePath, 'r+', encoding='utf-8') as f:
                    text = f.read()
                    strings = re.findall(r'def __init__[\w\W]+?Parameters\s+-+\s+([\w\W]+?)\s+Returns', text)

                if len(strings) > 0:
                    signatures = re.findall(r'def __init__\(([\w\W]+?)\):', text)
                    if len(signatures) == 0:
                        continue

                    signatureString = signatures[0].lstrip().rstrip().replace('\n', ' ')
                    signatureString = re.sub(r'\n\s+', r'\n', signatureString)
                    # signatures = signatures.split(', ')[1:]

                    argStrings = strings[0].replace('\n        ', '\n')
                    lines = argStrings.splitlines()
                    keys, docs = [], []
                    doc = ''
                    for line in lines:
                        if not line.startswith('    '):
                            keys.append(line.lstrip().rstrip().split(': ')[0])
                            if not doc == '':
                                docs.append(doc.rstrip())
                            doc = ''
                        else:
                            doc += line.lstrip().rstrip() + '\n'
                    docs.append(doc)
                    attr_docstring = '    Attributes\n    ----------'
                    atts = ''
                    for key, doc in zip(keys, docs):
                        doc = doc.rstrip()
                        att_doc = doc.replace('\n', '\n        ')
                        attr_docstring += f'\n    {key}\n        {att_doc}'

                        index = signatureString.index(key + ': ')
                        if key == keys[-1]:
                            signature = signatureString[index:]
                        else:
                            next_index = signatureString.index(keys[keys.index(key) + 1] + ': ')
                            signature = signatureString[index:next_index]

                        signature = signature.lstrip().rstrip()
                        if signature.endswith(','):
                            signature = signature[:-1]
                        att_doc = '# ' + doc.replace('\n', '\n    # ')
                        atts += f'\n\n    {att_doc}\n    {signature}'
                    atts = atts.lstrip()
                    new_text = text.replace('    Notes\n    -----',
                                            f'{attr_docstring}\n\n    Notes\n    -----') \
                        .replace('def __init__(', f'{atts}\n\n    def __init__(')
                    if '\\' in atts:
                        new_text = re.sub(r'(class \w+[\w\W]+?:\n\s+)"""', r'\g<1>r"""', new_text)
                    with open(pyFilePath, 'w+', encoding='utf-8') as f:
                        f.write(new_text)


# addAttributes('markdown')

def replaceNotes(searchDir: str):
    print(f'# Processing dir: {searchDir}')
    if not os.path.isdir(searchDir):
        return ''

    files = os.listdir(searchDir)
    for file in files:
        if os.path.isdir(os.path.join(searchDir, file)):
            replaceNotes(os.path.join(searchDir, file))
        elif os.path.isfile(os.path.join(searchDir, file)) and file.endswith('.py'):
            print(f'    # Processing file: {file}')
            filePath = os.path.join(searchDir, file)
            with open(filePath, 'r+', encoding='utf-8') as f:
                text = f.read()

            # Replace Class Notes
            class_notes = re.findall(r'(Notes\s+-----\s+This object can be accessed by:\s+\.\. code-block:: python\s+)'
                                     r'([\w\W]+?)\n\s+"""', text)
            for head, code in class_notes:
                new_head = '.. note:: \n        This object can be accessed by:\n\n        '\
                           '.. code-block:: python\n\n        '
                new_code = '    ' + code.replace('\n    ', '\n        ')
                string = head + code
                new_string = new_head + new_code
                text = text.replace(string, new_string)

            # Replace Method Notes
            method_notes = re.findall(r'(Notes\s+-----\s+This function can be accessed by:\s+\.\. code-block:: python\s+)'
                                      r'([\w\W]+?)\n\n', text)
            for head, code in method_notes:
                new_head = '.. note:: \n            This function can be accessed by:\n\n'\
                           '            .. code-block:: python\n\n            '
                new_code = '    ' + code.replace('\n    ', '\n        ')
                string = head + code
                new_string = new_head + new_code
                text = text.replace(string, new_string)

            if len(class_notes) + len(method_notes) > 0:
                with open(filePath, 'r+', encoding='utf-8') as f:
                    f.write(text)


replaceNotes('abaqus')



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class PEGLoadState(LoadState):
    """The PEGLoadState object stores the propagating data for a concentrated force in a step.
    One instance of this object is created internally by the PEGLoad object for each step.
    The instance is also deleted internally by the PEGLoad object.
    The PEGLoadState object has no constructor or methods.
    The PEGLoadState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - CLOAD
    """

    #: A Float or a Complex specifying the load component at dof 1 of reference node 1.
    comp1: Optional[float] = None

    #: A Float or a Complex specifying the load component at dof 1 of reference node 2.
    comp2: Optional[float] = None

    #: A Float or a Complex specifying the load component at dof 2 of reference node 2.
    comp3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the load component at dof 1 of
    #: reference node 1. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the load component at dof 1 of
    #: reference node 2. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the load component at dof 2 of
    #: reference node 2. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from __future__ import annotations

from typing import List, Tuple, Sequence, Dict

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .MeshElement import MeshElement
from ..BasicGeometry.EdgeArray import EdgeArray
from ..BasicGeometry.FaceArray import FaceArray


@abaqus_class_doc
class MeshElementArray(List[MeshElement]):
    """The MeshElementArray is a sequence of MeshElement objects.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].elements
            mdb.models[name].parts[name].allInternalSurfaces[name].elements
            mdb.models[name].parts[name].allSets[name].elements
            mdb.models[name].parts[name].allSurfaces[name].elements
            mdb.models[name].parts[name].elements
            mdb.models[name].parts[name].sets[name].elements
            mdb.models[name].parts[name].surfaces[name].elements
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].elements
            mdb.models[name].rootAssembly.allInstances[name].sets[name].elements
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name].elements
            mdb.models[name].rootAssembly.allInternalSets[name].elements
            mdb.models[name].rootAssembly.allInternalSurfaces[name].elements
            mdb.models[name].rootAssembly.allSets[name].elements
            mdb.models[name].rootAssembly.allSurfaces[name].elements
            mdb.models[name].rootAssembly.elements
            mdb.models[name].rootAssembly.instances[name].elements
            mdb.models[name].rootAssembly.instances[name].sets[name].elements
            mdb.models[name].rootAssembly.instances[name].surfaces[name].elements
            mdb.models[name].rootAssembly.modelInstances[i].elements
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].elements
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name].elements
            mdb.models[name].rootAssembly.sets[name].elements
            mdb.models[name].rootAssembly.surfaces[name].elements
    """

    @abaqus_method_doc
    def __init__(self, elements: List[MeshElement]) -> None:
        """This method creates a MeshElementArray object.

        .. note:: 
            This function can be accessed by::

                mesh.MeshElementArray

        Parameters
        ----------
        elements
            A list of MeshElement objects.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object.
        """
        super().__init__()

    @abaqus_method_doc
    def getFromLabel(self, label: int) -> MeshElement:
        """This method returns the object in the MeshElementArray with the given label.

        Parameters
        ----------
        label
            An Int specifying the label of the object.

        Returns
        -------
        MeshElement
            A :py:class:`~abaqus.Mesh.MeshElement.MeshElement` object.
        """
        ...

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: str) -> MeshElementArray:
        """This method returns the objects in the MeshElementArray identified using the specified
        **mask**. This command is generated when the JournalOptions are set to COMPRESSEDINDEX.
        When a large number of objects are involved, this method is highly efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object.
        """
        ...

    @abaqus_method_doc
    def getMask(self) -> str:
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingBox(
        self,
        xMin: float = ...,
        yMin: float = ...,
        zMin: float = ...,
        xMax: float = ...,
        yMax: float = ...,
        zMax: float = ...,
    ) -> MeshElementArray:
        """This method returns an array of element objects that lie within the specified bounding
        box.

        Parameters
        ----------
        xMin
            A float specifying the minimum X boundary of the bounding box.
        yMin
            A float specifying the minimum Y boundary of the bounding box.
        zMin
            A float specifying the minimum Z boundary of the bounding box.
        xMax
            A float specifying the maximum X boundary of the bounding box.
        yMax
            A float specifying the maximum Y boundary of the bounding box.
        zMax
            A float specifying the maximum Z boundary of the bounding box.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object, which is a sequence of MeshElement objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingCylinder(
        self, center1: tuple, center2: tuple, radius: str
    ) -> MeshElementArray:
        """This method returns an array of element objects that lie within the specified bounding
        cylinder.

        Parameters
        ----------
        center1
            A tuple of the X-, Y-, and Z-coordinates of the center of the first end of the cylinder.
        center2
            A tuple of the X-, Y-, and Z-coordinates of the center of the second end of the
            cylinder.
        radius
            A float specifying the radius of the cylinder.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object, which is a sequence of MeshElement objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingSphere(
        self, center: Tuple[float, float, float], radius: float
    ) -> MeshElementArray:
        """This method returns an array of element objects that lie within the specified bounding
        sphere.

        Parameters
        ----------
        center
            A tuple of the X-, Y-, and Z-coordinates of the center of the sphere.
        radius
            A float specifying the radius of the sphere.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object, which is a sequence of MeshElement objects.
        """
        ...

    @abaqus_method_doc
    def getBoundingBox(self) -> Dict[str, Tuple[float, float, float]]:
        """This method returns a dictionary of two tuples representing minimum and maximum boundary
        values of the bounding box of the minimum size containing the element sequence.

        Returns
        -------
        Dict[str, Tuple[float, float, float]]
            A Dictionary object with the following items:
            
            - **low**: a tuple of three floats representing the minimum x, y, and z boundary values of
              the bounding box.
            - **high**: a tuple of three floats representing the maximum x, y, and z boundary values of
              the bounding box.
        """
        ...

    @abaqus_method_doc
    def sequenceFromLabels(self, labels: Sequence[int]) -> MeshElementArray:
        """This method returns the objects in the MeshElementArray identified using the specified
        labels.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the labels.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object.

        Raises
        ------
        Error: The mask results in an empty sequence
            An exception occurs if the resulting sequence is empty.
        """
        ...

    @abaqus_method_doc
    def getExteriorEdges(self) -> EdgeArray:
        """This method returns the edges on the exterior of the faces in the FaceArray. That is, it
        returns the edges that are referenced by exactly one of the faces in the sequence.

        .. versionadded:: 2018
            The `getExteriorEdges` method was added.

        Returns
        -------
        EdgeArray
            An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object specifying the exterior edges.

        """
        ...

    @abaqus_method_doc
    def getExteriorFaces(self) -> FaceArray:
        """This method returns the cell faces on the exterior of the CellArray. That is, it returns
        the faces that are referenced by exactly one of the cells in the sequence.

        .. versionadded:: 2018
            The `getExteriorFaces` method was added.

        Returns
        -------
        FaceArray
            A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object representing the faces on the exterior of the cells.

        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConnectorBehaviorOption import ConnectorBehaviorOption
from ..UtilityAndView.abaqusConstants import ALL, SymbolicConstant


@abaqus_class_doc
class ConnectorFailure(ConnectorBehaviorOption):
    """The ConnectorFailure object defines failure criteria for one or more components of a
    connector's relative motion.
    The ConnectorFailure object is derived from the ConnectorBehaviorOption object.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name].behaviorOptions[i]
            import odbSection
            session.odbs[name].sections[name].behaviorOptions[i]

        The corresponding analysis keywords are:

        - CONNECTOR FAILURE
    """

    #: The SymbolicConstant ALL or an Int specifying the motion components that fail. If an Int
    #: is specified, only that motion component fails when the failure criteria are satisfied.
    #: If **releaseComponent** = ALL, all motion components fail. The default value is ALL.
    releaseComponent: SymbolicConstant = ALL

    #: None or a Float specifying the lower bound for the connector's relative position for all
    #: specified components, or no lower bound. The default value is None.
    minMotion: Optional[float] = None

    #: None or a Float specifying the upper bound for the connector's relative position for all
    #: specified components, or no upper bound. The default value is None.
    maxMotion: Optional[float] = None

    #: None or a Float specifying the lower bound of the force or moment in the directions of
    #: the specified components at which locking occurs, or no lower bound. The default value
    #: is None.
    minForce: Optional[float] = None

    #: None or a Float specifying the upper bound of the force or moment in the directions of
    #: the specified components at which locking occurs, or no upper bound. The default value
    #: is None.
    maxForce: Optional[float] = None

    #: A sequence of Ints specifying the components of relative motion for which the behavior
    #: is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
    #: specified. The default value is an empty sequence.
    components: tuple = ()

    @abaqus_method_doc
    def __init__(
        self,
        releaseComponent: SymbolicConstant = ALL,
        minMotion: Optional[float] = None,
        maxMotion: Optional[float] = None,
        minForce: Optional[float] = None,
        maxForce: Optional[float] = None,
        components: tuple = (),
    ):
        """This method creates a connector failure behavior option for a ConnectorSection object.

        .. note:: 
            This function can be accessed by::

                import connectorBehavior
                connectorBehavior.ConnectorFailure
                import odbConnectorBehavior
                odbConnectorBehavior.ConnectorFailure

        Parameters
        ----------
        releaseComponent
            The SymbolicConstant ALL or an Int specifying the motion components that fail. If an Int
            is specified, only that motion component fails when the failure criteria are satisfied.
            If **releaseComponent** = ALL, all motion components fail. The default value is ALL.
        minMotion
            None or a Float specifying the lower bound for the connector's relative position for all
            specified components, or no lower bound. The default value is None.
        maxMotion
            None or a Float specifying the upper bound for the connector's relative position for all
            specified components, or no upper bound. The default value is None.
        minForce
            None or a Float specifying the lower bound of the force or moment in the directions of
            the specified components at which locking occurs, or no lower bound. The default value
            is None.
        maxForce
            None or a Float specifying the upper bound of the force or moment in the directions of
            the specified components at which locking occurs, or no upper bound. The default value
            is None.
        components
            A sequence of Ints specifying the components of relative motion for which the behavior
            is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
            specified. The default value is an empty sequence.

        Returns
        -------
        ConnectorFailure
            A :py:class:`~abaqus.Connector.ConnectorFailure.ConnectorFailure` object.

        Raises
        ------
        ValueError and TextError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConnectorFailure object.

        Raises
        ------
        ValueError
        """
        ...



from xyPlot import AreaStyle 
from xyPlot import LineStyle 
from xyPlot import QuantityType 
from regionToolset import Region 
from sketch import Sketch 
from sketch import SketchTransform 
from xyPlot import SymbolStyle 
from xyPlot import TextStyle 
from xyPlot import XYData 
from xyPlot import XYDataFromFile 
from xyPlot import XYDataFromFreeBody 
from xyPlot import XYDataFromHistory 
from xyPlot import XYDataFromPath 
from xyPlot import XYDataFromShellThickness 
from xyPlot import xyDataListFromField 
# inspected from Abaqus cli



from typing import Dict, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AnalysisStep import AnalysisStep
from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import (Boolean, DEFAULT, OFF, SOLVER_DEFAULT, SUBSPACE,
                                              SymbolicConstant)


@abaqus_class_doc
class BuckleStep(AnalysisStep):
    """The BuckleStep object controls eigenvalue buckling estimation.
    The BuckleStep object is derived from the AnalysisStep object.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]

        The corresponding analysis keywords are:

        - BUCKLE
        - STEP
    """

    #: A String specifying the repository key.
    name: str = ""

    #: An Int specifying the number of eigenvalues to be estimated.
    numEigen: Optional[int] = None

    #: A SymbolicConstant specifying the eigensolver. Possible values are SUBSPACE and LANCZOS.
    #: The default value is SUBSPACE.
    eigensolver: SymbolicConstant = SUBSPACE

    #: None or a Float specifying the minimum eigenvalue of interest. The default value is
    #: None.
    minEigen: Optional[float] = None

    #: None or a Float specifying the maximum eigenvalue of interest. The default value is
    #: None.
    maxEigen: Optional[float] = None

    #: An Int specifying the number of vectors used in the iteration. The default value is the
    #: minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested.
    vectors: Optional[int] = None

    #: An Int specifying the maximum number of iterations. The default value is 30.
    maxIterations: int = 30

    #: The SymbolicConstant DEFAULT or an Int specifying the size of the Lanczos block steps.
    #: The default value is DEFAULT.
    blockSize: SymbolicConstant = DEFAULT

    #: The SymbolicConstant DEFAULT or an Int specifying the maximum number of Lanczos block
    #: steps within each Lanczos run. The default value is DEFAULT.Note:*minEigen*,
    #: **blockSize**, and **maxBlocks** are ignored unless **eigensolver** = LANCZOS.
    maxBlocks: SymbolicConstant = DEFAULT

    #: A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
    #: UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.
    matrixStorage: SymbolicConstant = SOLVER_DEFAULT

    #: A String specifying the name of the previous step. The new step appears after this step
    #: in the list of analysis steps.
    previous: str = ""

    #: A String specifying a description of the new step. The default value is an empty string.
    description: str = ""

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType* = ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: Optional[SymbolicConstant] = None

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #: 
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: Optional[SymbolicConstant] = None

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: Dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: Dict[str, HistoryOutputRequestState] = {}

    #: A :py:class:`~abaqus.StepOutput.DiagnosticPrint.DiagnosticPrint` object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A :py:class:`~abaqus.StepOutput.Monitor.Monitor` object.
    monitor: Optional[Monitor] = None

    #: A :py:class:`~abaqus.StepOutput.Restart.Restart` object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: Dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: Dict[str, AdaptiveMeshDomain] = {}

    #: A :py:class:`~abaqus.StepMiscellaneous.Control.Control` object.
    control: Control = Control()

    #: A :py:class:`~abaqus.StepMiscellaneous.SolverControl.SolverControl` object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: Dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: Optional[int] = None

    #: A repository of LoadState objects.
    loadStates: Dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: Dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: Dict[str, PredefinedFieldState] = {}

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        previous: str,
        numEigen: int,
        description: str = "",
        eigensolver: SymbolicConstant = SUBSPACE,
        minEigen: Optional[float] = None,
        maxEigen: Optional[float] = None,
        vectors: Optional[int] = None,
        maxIterations: int = 30,
        blockSize: SymbolicConstant = DEFAULT,
        maxBlocks: SymbolicConstant = DEFAULT,
        matrixStorage: SymbolicConstant = SOLVER_DEFAULT,
        maintainAttributes: Boolean = False,
    ):
        """This method creates a BuckleStep object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].BuckleStep

        Parameters
        ----------
        name
            A String specifying the repository key.
        previous
            A String specifying the name of the previous step. The new step appears after this step
            in the list of analysis steps.
        numEigen
            An Int specifying the number of eigenvalues to be estimated.
        description
            A String specifying a description of the new step. The default value is an empty string.
        eigensolver
            A SymbolicConstant specifying the eigensolver. Possible values are SUBSPACE and LANCZOS.
            The default value is SUBSPACE.
        minEigen
            None or a Float specifying the minimum eigenvalue of interest. The default value is
            None.
        maxEigen
            None or a Float specifying the maximum eigenvalue of interest. The default value is
            None.
        vectors
            An Int specifying the number of vectors used in the iteration. The default value is the
            minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested.
        maxIterations
            An Int specifying the maximum number of iterations. The default value is 30.
        blockSize
            The SymbolicConstant DEFAULT or an Int specifying the size of the Lanczos block steps.
            The default value is DEFAULT.
        maxBlocks
            The SymbolicConstant DEFAULT or an Int specifying the maximum number of Lanczos block
            steps within each Lanczos run. The default value is DEFAULT.Note:*minEigen*,
            **blockSize**, and **maxBlocks** are ignored unless **eigensolver** = LANCZOS.
        matrixStorage
            A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
            UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.
        maintainAttributes
            A Boolean specifying whether to retain attributes from an existing step with the same
            name. The default value is False.

        Returns
        -------
        BuckleStep
            A :py:class:`~abaqus.Step.BuckleStep.BuckleStep` object.

        Raises
        ------
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        description: str = "",
        eigensolver: SymbolicConstant = SUBSPACE,
        minEigen: Optional[float] = None,
        maxEigen: Optional[float] = None,
        vectors: Optional[int] = None,
        maxIterations: int = 30,
        blockSize: SymbolicConstant = DEFAULT,
        maxBlocks: SymbolicConstant = DEFAULT,
        matrixStorage: SymbolicConstant = SOLVER_DEFAULT,
    ):
        """This method modifies the BuckleStep object.

        Parameters
        ----------
        description
            A String specifying a description of the new step. The default value is an empty string.
        eigensolver
            A SymbolicConstant specifying the eigensolver. Possible values are SUBSPACE and LANCZOS.
            The default value is SUBSPACE.
        minEigen
            None or a Float specifying the minimum eigenvalue of interest. The default value is
            None.
        maxEigen
            None or a Float specifying the maximum eigenvalue of interest. The default value is
            None.
        vectors
            An Int specifying the number of vectors used in the iteration. The default value is the
            minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested.
        maxIterations
            An Int specifying the maximum number of iterations. The default value is 30.
        blockSize
            The SymbolicConstant DEFAULT or an Int specifying the size of the Lanczos block steps.
            The default value is DEFAULT.
        maxBlocks
            The SymbolicConstant DEFAULT or an Int specifying the maximum number of Lanczos block
            steps within each Lanczos run. The default value is DEFAULT.Note:*minEigen*,
            **blockSize**, and **maxBlocks** are ignored unless **eigensolver** = LANCZOS.
        matrixStorage
            A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
            UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.

        Raises
        ------
        RangeError
        """
        ...



from typing import Optional, Union

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import (ACTIVE_CUT_RANGE, Boolean, CENTROID,
                                              CURRENT_DISPLAY_GROUP, GLOBAL, NORMAL_TANGENTIAL, OFF,
                                              ON, SymbolicConstant)
from .._OptionsBase import _CopyOptionsBase


@abaqus_class_doc
class ViewCutOptions(_CopyOptionsBase):
    """The ViewCutOptions object stores values and attributes associated with a view cut plot.
    The ViewCutOptions object has no constructor command. Abaqus creates a
    *defaultOdbDisplay.viewCutOptions* member when you import the Visualization module.
    Abaqus creates an **viewCutOptions** member when it creates the OdbDisplay object, using
    the values from *defaultOdbDisplay.viewCutOptions*. Abaqus creates the **odbDisplay**
    member when a viewport is created, using the values from **defaultOdbDisplay**.
    ViewCutOptions objects are accessed in one of two ways:
    - The default view cut options. These settings are used as defaults when other
    **viewCutOptions** members are created. These settings can be set to customize user
    preferences.
    - The view cut options associated with a particular viewport.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.defaultOdbDisplay.viewCutOptions
            session.viewports[name].layers[name].odbDisplay.viewCutOptions
            session.viewports[name].odbDisplay.viewCutOptions
    """

    #: A Boolean specifying whether to use the options defined for displaying the model below
    #: the cut. The default value is OFF.
    useBelowOptions: Boolean = OFF

    #: A Boolean specifying whether to use the options defined for displaying the model on the
    #: cut. The default value is OFF.
    useOnOptions: Boolean = OFF

    #: A Boolean specifying whether to use the options defined for displaying the model above
    #: the cut. The default value is OFF.
    useAboveOptions: Boolean = OFF

    #: An Int specifying number of free bodies per view cut. The default value is 1.
    numCutFreeBody: int = 1

    #: A Boolean specifying whether to display slicing. The default value is OFF.
    displaySlicing: Boolean = OFF

    #: A Boolean specifying whether to slice at path nodes. The default value is OFF.
    slicingAtPathNodes: Boolean = OFF

    #: A Boolean specifying whether to put the summation point at the path. The default value
    #: is ON.
    freeBodySumOnPath: Boolean = ON

    #: A Float specifying free body minimum value. The default value is 0.0.
    cutFreeBodyMin: float = 0

    #: A Float specifying free body maximum value. The default value is 0.0.
    cutFreeBodyMax: float = 0

    #: A SymbolicConstant specifying the domain through which the free body cuts. Possible
    #: values are CURRENT_DISPLAY_GROUP and WHOLE_MODEL. The default value is
    #: CURRENT_DISPLAY_GROUP.
    freeBodyCutThru: SymbolicConstant = CURRENT_DISPLAY_GROUP

    #: A SymbolicConstant specifying the domain through which the free body steps. Possible
    #: values are ACTIVE_CUT_RANGE and PREDEFINED_PATH. The default value is ACTIVE_CUT_RANGE.
    freeBodyStepThru: SymbolicConstant = ACTIVE_CUT_RANGE

    #: A SymbolicConstant specifying whether to show the heat flow rate when available.
    #: Possible values are ON and OFF. The default value is ON.
    showHeatFlowRate: Union[SymbolicConstant, Boolean] = ON

    #: A SymbolicConstant specifying the summation location for the free body cut. Possible
    #: values are CENTROID and SPECIFY. The default value is CENTROID.
    summationLoc: SymbolicConstant = CENTROID

    #: A SymbolicConstant specifying the component resolution choice for the free body cut.
    #: Possible values are NORMAL_TANGENTIAL and CSYS. The default value is NORMAL_TANGENTIAL.
    componentResolution: SymbolicConstant = NORMAL_TANGENTIAL

    #: None or an OptionArg object specifying values to be used for defining the options
    #: applicable on the model below the cut. The default value is None.
    belowOptions: Optional[str] = None

    #: None or an OptionArg object specifying values to be used for defining the options
    #: applicable on the model on the cut. The default value is None.
    onOptions: Optional[str] = None

    #: None or an OptionArg object specifying values to be used for defining the options
    #: applicable on the model above the cut. The default value is None.
    aboveOptions: Optional[str] = None

    #: The SymbolicConstant GLOBAL or a String specifying the coordinate system name for the
    #: free body cut's component resolution. The default value is GLOBAL.
    csysName: SymbolicConstant = GLOBAL

    #: A String specifying the name of the path along which slicing occurs. The default value
    #: is an empty string.
    pathName: str = ""

    #: A tuple of three Floats specifying the summation point for the free body cut. The
    #: default value is (0, 0, 0).
    summationPoint: Optional[float] = None

    #: A tuple of three Floats specifying the Y axis for free body component resolution. The
    #: default value is (0, 1, 0).
    yAxis: Optional[float] = None

    @abaqus_method_doc
    def setValues(
        self,
        options: Optional["ViewCutOptions"] = None,
        *,
        belowOptions: Optional[str] = None,
        useBelowOptions: Boolean = OFF,
        onOptions: Optional[str] = None,
        useOnOptions: Boolean = OFF,
        aboveOptions: Optional[str] = None,
        useAboveOptions: Boolean = OFF,
        freeBodyCutThru: SymbolicConstant = CURRENT_DISPLAY_GROUP,
        freeBodyStepThru: SymbolicConstant = ACTIVE_CUT_RANGE,
        numCutFreeBody: int = 1,
        displaySlicing: Boolean = OFF,
        slicingAtPathNodes: Boolean = OFF,
        freeBodySumOnPath: Boolean = ON,
        cutFreeBodyMin: float = 0,
        cutFreeBodyMax: float = 0,
        showHeatFlowRate: Union[SymbolicConstant, Boolean] = ON,
        summationLoc: SymbolicConstant = CENTROID,
        componentResolution: SymbolicConstant = NORMAL_TANGENTIAL,
        csysName: SymbolicConstant = GLOBAL,
        pathName: str = "",
        summationPoint: tuple = (),
        yAxis: tuple = (),
    ):
        """This method modifies the ViewCutOptions object.

        Parameters
        ----------
        options
            A :py:class:`~abaqus.PlotOptions.ViewCutOptions.ViewCutOptions` object from which values are to be copied.
            If other arguments are also supplied to setValues, they will override the values in **options**. The default
            value is None.
        belowOptions
            None or an OptionArg object specifying values to be used for defining the options
            applicable on the model below the cut. The default value is None.
        useBelowOptions
            A Boolean specifying whether to use the options defined for displaying the model below
            the cut. The default value is OFF.
        onOptions
            None or an OptionArg object specifying values to be used for defining the options
            applicable on the model on the cut. The default value is None.
        useOnOptions
            A Boolean specifying whether to use the options defined for displaying the model on the
            cut. The default value is OFF.
        aboveOptions
            None or an OptionArg object specifying values to be used for defining the options
            applicable on the model above the cut. The default value is None.
        useAboveOptions
            A Boolean specifying whether to use the options defined for displaying the model above
            the cut. The default value is OFF.
        freeBodyCutThru
            A SymbolicConstant specifying the domain through which the free body cuts. Possible
            values are CURRENT_DISPLAY_GROUP and WHOLE_MODEL. The default value is
            CURRENT_DISPLAY_GROUP.
        freeBodyStepThru
            A SymbolicConstant specifying the domain through which the free body steps. Possible
            values are ACTIVE_CUT_RANGE and PREDEFINED_PATH. The default value is ACTIVE_CUT_RANGE.
        numCutFreeBody
            An Int specifying number of free bodies per view cut. The default value is 1.
        displaySlicing
            A Boolean specifying whether to display slicing. The default value is OFF.
        slicingAtPathNodes
            A Boolean specifying whether to slice at path nodes. The default value is OFF.
        freeBodySumOnPath
            A Boolean specifying whether to put the summation point at the path. The default value
            is ON.
        cutFreeBodyMin
            A Float specifying free body minimum value. The default value is 0.0.
        cutFreeBodyMax
            A Float specifying free body maximum value. The default value is 0.0.
        showHeatFlowRate
            A SymbolicConstant specifying whether to show the heat flow rate when available.
            Possible values are ON and OFF. The default value is ON.
        summationLoc
            A SymbolicConstant specifying the summation location for the free body cut. Possible
            values are CENTROID and SPECIFY. The default value is CENTROID.
        componentResolution
            A SymbolicConstant specifying the component resolution choice for the free body cut.
            Possible values are NORMAL_TANGENTIAL and CSYS. The default value is NORMAL_TANGENTIAL.
        csysName
            The SymbolicConstant GLOBAL or a String specifying the coordinate system name for the
            free body cut's component resolution. The default value is GLOBAL.
        pathName
            A String specifying the name of the path along which slicing occurs. The default value
            is an empty string.
        summationPoint
            A sequence of three Floats specifying the summation point for the free body cut. The
            default value is (0, 0, 0).
        yAxis
            A sequence of three Floats specifying the Y axis for free body component resolution. The
            default value is (0, 1, 0).

        Raises
        ------
        RangeError
        """
        super().setValues(
            options=options,
            aboveOptions=aboveOptions,
            belowOptions=belowOptions,
            componentResolution=componentResolution,
            csysName=csysName,
            cutFreeBodyMax=cutFreeBodyMax,
            cutFreeBodyMin=cutFreeBodyMin,
            displaySlicing=displaySlicing,
            freeBodyCutThru=freeBodyCutThru,
            freeBodyStepThru=freeBodyStepThru,
            freeBodySumOnPath=freeBodySumOnPath,
            numCutFreeBody=numCutFreeBody,
            onOptions=onOptions,
            pathName=pathName,
            showHeatFlowRate=showHeatFlowRate,
            slicingAtPathNodes=slicingAtPathNodes,
            summationLoc=summationLoc,
            summationPoint=summationPoint,
            useAboveOptions=useAboveOptions,
            useBelowOptions=useBelowOptions,
            useOnOptions=useOnOptions,
            yAxis=yAxis,
        )



from typing import Union, overload, Dict, List, Optional, Tuple, Sequence

from abaqus.Datum.DatumCsys import DatumCsys
from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AssemblyFeature import AssemblyFeature
from .AssemblyModel import AssemblyModel
from .ConnectorOrientationArray import ConnectorOrientationArray
from .ModelInstance import ModelInstance
from .PartInstance import PartInstance
from ..BasicGeometry.EdgeArray import EdgeArray
from ..BasicGeometry.Face import Face
from ..BasicGeometry.ReferencePoint import ReferencePoint
from ..BasicGeometry.VertexArray import VertexArray
from ..Datum.Datum import Datum
from ..EngineeringFeature.EngineeringFeature import EngineeringFeature
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshElementArray import MeshElementArray
from ..Mesh.MeshFace import MeshFace
from ..Mesh.MeshNode import MeshNode
from ..Mesh.MeshNodeArray import MeshNodeArray
from ..Part.Part import Part
from ..Property.SectionAssignmentArray import SectionAssignmentArray
from ..Region.Set import Set
from ..Region.Skin import Skin
from ..Region.Stringer import Stringer
from ..Region.Surface import Surface
from ..UtilityAndView.abaqusConstants import ALL_EDGES, Boolean, LOW, OFF, ON, SymbolicConstant


@abaqus_class_doc
class AssemblyBase(AssemblyFeature):
    """An :py:class:`~abaqus.Assembly.Assembly.Assembly` object is a container for instances of parts. The Assembly object has no
    constructor command. Abaqus creates the **rootAssembly** member when a Model object is
    created.

    .. note:: 
        This object can be accessed by::

            import assembly
            mdb.models[name].rootAssembly
    """

    #: An Int specifying that feature parameters have been modified but that the assembly has
    #: not been regenerated. Possible values are 0 and 1.
    isOutOfDate: Optional[int] = None

    #: A Float specifying which gives an indication when the assembly was last modified.
    timeStamp: Optional[float] = None

    #: An Int specifying whether the assembly is locked or not. Possible values are 0 and 1.
    isLocked: Optional[int] = None

    #: A Boolean specifying whether the positioning constraints in the assembly should be
    #: regenerated together before regenerating other assembly features. The default value is
    #: ON.If the assembly has position constraint features and you modify the value of
    #: **regenerateConstraintsTogether**, Abaqus/CAE will regenerate the assembly features.
    regenerateConstraintsTogether: Boolean = ON

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object specifying all the vertices existing at the assembly level. This
    #: member does not provide access to the vertices at the instance level.
    vertices: VertexArray = VertexArray([])

    #: An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object specifying all the edges existing at the assembly level. This member
    #: does not provide access to the edges at the instance level.
    edges: EdgeArray = EdgeArray([])

    #: A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object specifying all the elements existing at the assembly level.
    #: This member does not provide access to the elements at the instance level.
    elements: MeshElementArray = MeshElementArray([])

    #: A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object specifying all the nodes existing at the assembly level. This
    #: member does not provide access to the nodes at the instance level.
    nodes: MeshNodeArray = MeshNodeArray([])

    #: A repository of PartInstance objects.
    instances: Dict[str, PartInstance] = {}

    #: A repository of Datum objects specifying all Datum objects in the assembly.
    datums: List[Datum] = []

    #: A repository of Feature objects specifying all Feature objects in the assembly.
    features: Dict[str, AssemblyFeature] = {}

    #: A repository of Feature objects specifying all Feature objects in the assembly.The
    #: Feature objects in the featuresById repository are the same as the Feature objects in
    #: the features repository. However, the key to the objects in the featuresById repository
    #: is an integer specifying the **ID**, whereas the key to the objects in the features
    #: repository is a string specifying the **name**.
    featuresById: Dict[str, AssemblyFeature] = {}

    #: A repository of Surface objects specifying for more information, see [Region
    #: commands](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAEKERRefMap/simaker-m-RegPyc-sb.htm?ContextScope=all).
    surfaces: Dict[str, Surface] = {}

    #: A repository of Surface objects specifying for more information, see [Region
    #: commands](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAEKERRefMap/simaker-m-RegPyc-sb.htm?ContextScope=all).
    allSurfaces: Dict[str, Surface] = {}

    #: A repository of Surface objects specifying picked regions.
    allInternalSurfaces: Dict[str, Surface] = {}

    #: A repository of Set objects.
    sets: Dict[str, Set] = {}

    #: A repository of Set objects specifying for more information, see [Region
    #: commands](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAEKERRefMap/simaker-m-RegPyc-sb.htm?ContextScope=all).
    allSets: Dict[str, Set] = {}

    #: A repository of Set objects specifying picked regions.
    allInternalSets: Dict[str, Set] = {}

    #: A repository of Skin objects specifying the skins created on the assembly.
    skins: Dict[str, Skin] = {}

    #: A repository of Stringer objects specifying the stringers created on the assembly.
    stringers: Dict[str, Stringer] = {}

    #: A repository of ReferencePoint objects.
    referencePoints: Dict[str, ReferencePoint] = {}

    #: A repository of ModelInstance objects.
    modelInstances: Dict[str, ModelInstance] = {}

    #: A :py:class:`~abaqus.Assembly.PartInstance.PartInstance` object specifying the PartInstances and A :py:class:`~abaqus.Assembly.ModelInstance.ModelInstance` object specifying
    #: the ModelInstances.
    allInstances: Dict[str, Union[PartInstance, ModelInstance]] = {}

    #: An :py:class:`~abaqus.EngineeringFeature.EngineeringFeature.EngineeringFeature` object.
    engineeringFeatures: EngineeringFeature = EngineeringFeature()

    #: A String specifying the name of the model to which the assembly belongs.
    modelName: str = ""

    #: A :py:class:`~abaqus.Assembly.ConnectorOrientationArray.ConnectorOrientationArray` object.
    connectorOrientations: ConnectorOrientationArray = []

    #: A :py:class:`~abaqus.Property.SectionAssignmentArray.SectionAssignmentArray` object.
    sectionAssignments: SectionAssignmentArray = []

    @overload
    @abaqus_method_doc
    def Instance(
        self, name: str, part: Part, autoOffset: Boolean = OFF, dependent: Boolean = OFF
    ) -> PartInstance:
        """This method creates a PartInstance object and puts it into the instances repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].rootAssembly.Instance

        Parameters
        ----------
        name
            A String specifying the repository key. The name must be a valid Abaqus object name.
        part
            A :py:class:`~abaqus.Part.Part.Part` object to be instanced. If the part does not exist, no PartInstance object is
            created.
        autoOffset
            A Boolean specifying whether to apply an auto offset to the new part instance that will
            offset it from existing part instances. The default value is OFF.
        dependent
            A Boolean specifying whether the part instance is dependent or independent. If
            **dependent** = OFF, the part instance is independent. The default value is OFF.

        Returns
        -------
        PartInstance
            A :py:class:`~abaqus.Assembly.PartInstance.PartInstance` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def Instance(
        self, name: str, model: AssemblyModel, autoOffset: Boolean = OFF
    ) -> ModelInstance:
        """This method creates a ModelInstance object and puts it into the instances repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].rootAssembly.Instance

        Parameters
        ----------
        name
            The repository key. The name must be a valid Abaqus object name.
        model
            A :py:class:`~abaqus.Model.Model.Model` object to be instanced. If the model does not exist, no ModelInstance object is
            created.
        autoOffset
            A Boolean specifying whether to apply an auto offset to the new instance that will
            offset it from existing instances. The default value is OFF.

        Returns
        -------
        ModelInstance
            A :py:class:`~abaqus.Assembly.ModelInstance.ModelInstance` object.
        """
        ...

    @abaqus_method_doc
    def Instance(
        self, name: str, *args, **kwargs
    ) -> Union[PartInstance, ModelInstance]:
        """This method creates a PartInstance object and puts it into the instances repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].rootAssembly.Instance

        Parameters
        ----------
        name
            A String specifying the repository key. The name must be a valid Abaqus object name.
        kwargs
            Key-value arguments

        Returns
        -------
        PartInstance
            A :py:class:`~abaqus.Assembly.PartInstance.PartInstance` object.
        """
        if "part" in kwargs.keys() or (len(args) > 0 and isinstance(args[0], Part)):
            instance = PartInstance(name, *args, **kwargs)
        else:
            instance = ModelInstance(name, *args, **kwargs)
            self.modelInstances[name] = instance
        self.instances[name] = instance
        self.allInstances[name] = instance
        return instance

    @abaqus_method_doc
    def backup(self):
        """This method makes a backup copy of the features in the assembly. The backup() method is
        used in conjunction with the restore() method.
        """
        ...

    @abaqus_method_doc
    def clearGeometryCache(self):
        """This method deletes the geometry cache. Deleting the geometry cache reduces the amount
        of memory being used.
        """
        ...

    @abaqus_method_doc
    def deleteAllFeatures(self):
        """This method deletes all the features in the assembly."""
        ...

    @abaqus_method_doc
    def deleteFeatures(self, featureNames: tuple):
        """This method deletes specified features from the assembly.

        Parameters
        ----------
        featureNames
            A sequence of Strings specifying the feature names that will be deleted from the
            assembly.
        """
        ...

    @abaqus_method_doc
    def excludeFromSimulation(self, instances: Sequence[PartInstance], exclude: str):
        """This method excludes the specified part instances from the analysis.

        Parameters
        ----------
        instances
            A sequence of PartInstance objects to be excluded from the analysis.
        exclude
            A Bool specifying whether to exclude the selected instances from the analysis or include
            them.
        """
        ...

    @abaqus_method_doc
    def featurelistInfo(self):
        """This method prints the name and status of all the features in the feature lists."""
        ...

    @abaqus_method_doc
    def getMassProperties(
        self,
        regions: str = "",
        relativeAccuracy: SymbolicConstant = LOW,
        useMesh: Boolean = False,
        specifyDensity: Boolean = False,
        density: str = "",
        specifyThickness: Boolean = False,
        thickness: str = "",
        miAboutCenterOfMass: Boolean = True,
        miAboutPoint: Tuple[float, float, float] = (),
    ):
        """This method returns the mass properties of the assembly, or instances or regions. Only
        beams, trusses, shells, solids, point, nonstructural mass, and rotary inertia elements
        are supported.

        Parameters
        ----------
        regions
            A MeshElementArray, CellArray, FaceArray, EdgeArray, or list of PartInstance objects
            specifying the regions whose mass properties are to be queried. The whole assembly is
            queried by default.
        relativeAccuracy
            A SymbolicConstant specifying the relative accuracy for geometry computation. Possible
            values are LOW, MEDIUM, and HIGH. The default value is LOW.
        useMesh
            A Boolean specifying whether the mesh should be used in the computation if the geometry
            is meshed. The default value is False.
        specifyDensity
            A Boolean specifying whether a user-specified density should be used in regions with
            density errors such as undefined material density. The default value is False.
        density
            A double value specifying the user-specified density value to be used in regions with
            density errors. The user-specified density should be greater than 0.
        specifyThickness
            A Boolean specifying whether a user-specified thickness should be used in regions with
            thickness errors such as undefined thickness. The default value is False.
        thickness
            A double value specifying the user-specified thickness value to be used in regions with
            thickness errors. The user-specified thickness should be greater than 0.
        miAboutCenterOfMass
            A Boolean specifying if the moments of inertia should be evaluated about the center of
            mass. The default value is True.
        miAboutPoint
            A tuple of three floats specifying the coordinates of the point about which to evaluate
            the moment of inertia. By default if the moments of inertia are not being evaluated
            about the center of mass, they will be evaluated about the origin.

        Returns
        -------
        properties: dict
            A Dictionary object with the following items:
            **area**: None or a Float specifying the sum of the area of the specified faces. The area
            is computed only for one side for shells.
            **areaCentroid**: None or a tuple of three Floats representing the coordinates of the area
            centroid.
            **volume**: None or a Float specifying the volume of the specified regions.
            **volumeCentroid**: None or a tuple of three Floats representing the coordinates of the
            volume centroid.
            **massFromMassPerUnitSurfaceArea**: None or a Float specifying the mass due to mass per
            unit surface area.
            **mass**: None or a Float specifying the mass of the specified regions. It is the total
            mass and includes mass from quantities such as mass per unit surface area.
            **centerOfMass**: None or a tuple of three Floats representing the coordinates of the
            center of mass.
            **momentOfInertia**: None or a tuple of six Floats representing the moments of inertia
            about the center of mass or about the point specified.
            **warnings**: A tuple of SymbolicConstants representing the problems encountered while
            computing the mass properties. Possible SymbolicConstants are:
            UNSUPPORTED_ENTITIES: Some unsupported entities exist in the specified regions. The mass
            properties are computed only for beams, trusses, shells, solids, point and
            non-structural mass elements, and rotary inertia elements. The mass properties are not
            computed for axisymmetric elements, springs, connectors, gaskets, or any other elements.
            MISSING_THICKNESS: For some regions, the section definitions are missing thickness
            values.
            ZERO_THICKNESS: For some regions, the section definitions have a zero thickness value.
            VARIABLE_THICKNESS: The nodal thickness or field thickness specified for some regions
            has been ignored.
            NON_APPLICABLE_THICKNESS: For some regions, the thickness value is not applicable to the
            corresponding sections specified on the regions.
            MISSING_DENSITY: For some regions, the section definitions are missing material density
            values.
            MISSING_MATERIAL_DEFINITION: For some regions, the material definition is missing.
            ZERO_DENSITY: For some regions, the section definitions have a zero material density
            value.
            UNSUPPORTED_DENSITY: For some regions, either a negative material density or a
            temperature dependent density has been specified, or the material value is missing for
            one or more plies in the composite section.
            SHELL_OFFSETS: For shells, this method does not account for any offsets specified.
            MISSING_SECTION_DEFINITION: For some regions, the section definition is missing.
            UNSUPPORTED_SECTION_DEFINITION: The section definition provided for some regions is not
            supported.
            REINFORCEMENTS: This method does not account for any reinforcements specified on the
            model.
            SMEARED_PROPERTIES: For regions with composite section assignments, the density is
            smeared across the thickness. The volume centroid and center of mass computations for a
            composite shell use a lumped mass approach where the volume and mass is assumed to be
            lumped in the plane of the shell. As a result of these approximations the volume
            centroid, center of mass and moments of inertia may be slightly inaccurate for regions
            with composite section assignments.
            UNSUPPORTED_NON_STRUCTURAL_MASS_ENTITIES: This method does not account for any
            non-structural mass on wires.
            INCORRECT_MOMENT_OF_INERTIA: For geometry regions with non-structural mass per volume,
            the non-structural mass is assumed to be a point mass at the centroid of the regions.
            Thus, the moments of inertia may be inaccurate as the distribution of the non-structural
            mass is not accounted for. Use the mesh for accurately computing the moments of inertia.
            MISSING_BEAM_ORIENTATIONS: For some regions with beam section assignments, the beam
            section orientations are missing.
            UNSUPPORTED_BEAM_PROFILES: This method supports the Box, Pipe, Circular, Rectangular,
            Hexagonal, Trapezoidal, I, L, T, Arbitrary, and Tapered beam profiles. Any other beam
            profile is not supported.
            TAPERED_BEAM_MI: Moment of inertia calculations for tapered beams are not accurate.
            SUBSTRUCTURE_INCORRECT_PROPERTIES: The user assigned density and thickness is not
            considered for substructures.
        """
        ...

    @abaqus_method_doc
    def getAngle(
        self, plane1: str, plane2: str, line1: str, line2: str, commonVertex: str = ""
    ):
        """This method returns the angle between the specified entities.

        Parameters
        ----------
        plane1
            A Face, MeshFace, or a Datum object specifying the first plane. The Datum object must
            represent a datum plane. The **plane1** and **line1** arguments are mutually exclusive. One
            of them must be specified.
        plane2
            A Face, MeshFace, or a Datum object specifying the second plane. The Datum object must
            represent a datum plane. The **plane2** and **line2** arguments are mutually exclusive. One
            of them must be specified.
        line1
            An Edge, MeshEdge, or a Datum object specifying the first curve. The Datum object must
            represent a datum axis. The **plane1** and **line1** arguments are mutually exclusive. One
            of them must be specified.
        line2
            An Edge, MeshEdge, or a Datum object specifying the second curve. The Datum object must
            represent a datum axis. The **plane2** and **line2** arguments are mutually exclusive. One
            of them must be specified.
        commonVertex
            If the two selected Edge objects have more than one vertex in common, this ConstrainedSketchVertex object
            specifies the vertex at which to evaluate the angle.

        Returns
        -------
        float
            A Float specifying the angle between the specified entities. If you provide a plane as
            an argument, Abaqus/CAE computes the angle using the normal to the plane.
        """
        ...

    @abaqus_method_doc
    def getCoordinates(self, entity: str):
        """This method returns the coordinates of a specified point.

        Parameters
        ----------
        entity
            A ConstrainedSketchVertex, Datum point, MeshNode, or ReferencePoint specifying the entity to query.

        Returns
        -------
        Tuple[float, float]
            A tuple of three Floats representing the coordinates of the specified point.
        """
        ...

    @abaqus_method_doc
    def getDistance(self, entity1: str, entity2: str, printResults: Boolean = OFF, csys: DatumCsys = DatumCsys()):
        """Depending on the arguments provided, this method returns one of the following:
        
        - The distance between two points.
        - The minimum distance between a point and an edge.
        - The minimum distance between two edges.

        Parameters
        ----------
        entity1
            A ConstrainedSketchVertex, Datum point, MeshNode, or Edge specifying the first entity from which to
            measure.
        entity2
            A ConstrainedSketchVertex, Datum point, MeshNode, or Edge specifying the second entity to which to
            measure.
        printResults
            A Boolean that determines whether a verbose output is to be printed. The default is True
        csys
            A DatumCsys object specifying the desired coordinate system of the returned coordinates.
            By default, coordinates are given in the global coordinate system.

            .. versionadded:: 2022
                The `csys` argument was added.

        Returns
        -------
        float
            A Float specifying the calculated distance.
        """
        ...

    @abaqus_method_doc
    def getFacesAndVerticesOfAttachmentLines(self, edges: EdgeArray):
        """Given an array of edge objects, this method returns a tuple of dictionary objects. Each
        object consists of five members including the attachment line and associated face and
        vertex objects.

        Parameters
        ----------
        edges
            An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object which is a sequence of Edge objects.

        Returns
        -------
        Sequence[dict]
            A tuple of dictionary objects. Each dictionary contains five items with the following keys:
        
            - **edge**: An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the attachment line.
            - **startFace**: A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the face associated with one end of the 
              attachment line.
            - **endFace**: A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the face associated with the other end of 
              the attachment line.
            - **startVertex**: A :py:class:`~abaqus.Sketcher.ConstrainedSketchVertex.ConstrainedSketchVertex.ConstrainedSketchVertex` 
              object specifying the vertex associated with one end of the attachment line. This end is also associated with the startFace.
            - **endVertex**: A :py:class:`~abaqus.Sketcher.ConstrainedSketchVertex.ConstrainedSketchVertex.ConstrainedSketchVertex` 
              object specifying the vertex associated with the other end of the attachment line. This end is also associated with the endFace.
        """
        ...

    @abaqus_method_doc
    def getSurfaceSections(self, surface: str):
        """This method returns a list of the sections assigned to the regions encompassed by the
        specified surface.

        Parameters
        ----------
        surface
            A string specifying the Surface name.

        Returns
        -------
        Sequence[str]
            A tuple of strings representing the section names. If no section names are found, the
            tuple will contain one empty string.
        """
        ...

    @abaqus_method_doc
    def importEafFile(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from an EAF file into the root assembly.

        Parameters
        ----------
        filename
            A String specifying the path to the EAF file from which to import the assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def importParasolidFile(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from the Parasolid file into the root assembly.

        Parameters
        ----------
        filename
            A String specifying the path to a Parasolid file from which to import the assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def importCatiaV5File(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from a CATIA V5 Elysium Neutral file into the root
        assembly.

        Parameters
        ----------
        filename
            A String specifying the path to the CATIA V5 Elysium Neutral file from which to import
            the assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def importEnfFile(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from an Elysium Neutral file created by Pro/ENGINEER,
        I-DEAS, or CATIA V5 into the root assembly.

        Parameters
        ----------
        filename
            A String specifying the path to the Elysium Neutral file from which to import the
            assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def importIdeasFile(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from an I-DEAS Elysium Neutral file into the root
        assembly.

        Parameters
        ----------
        filename
            A String specifying the path to the I-DEAS Elysium Neutral file from which to import the
            assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def importProEFile(self, filename: str, ids: tuple = ()):
        """This method imports an assembly from a Pro/ENGINEER Elysium Neutral file into the root
        assembly.

        Parameters
        ----------
        filename
            A String specifying the path to the Pro/ENGINEER Elysium Neutral file from which to
            import the assembly.
        ids
            A sequence of Ints. Each Int in the sequence is a unique identifier of the occurrence in
            the assembly tree or component identifier associated with the part in the EAF file. If
            **ids** is an empty sequence, all occurrences or parts will be imported. The default value
            is an empty sequence.
        """
        ...

    @abaqus_method_doc
    def makeDependent(self, instances: Sequence[PartInstance]):
        """This method converts the specified part instances from independent to dependent part
        instances.

        Parameters
        ----------
        instances
            A sequence of PartInstance objects to convert to dependent part instances.
        """
        ...

    @abaqus_method_doc
    def makeIndependent(self, instances: Sequence[PartInstance]):
        """This method converts the specified part instances from dependent to independent part
        instances.

        Parameters
        ----------
        instances
            A sequence of PartInstance objects to convert to independent part instances.
        """
        ...

    @abaqus_method_doc
    def printAssignedSections(self):
        """This method prints a summary of assigned connector sections."""
        ...

    @abaqus_method_doc
    def printConnectorOrientations(self):
        """This method prints a summary of connector orientations."""
        ...

    @abaqus_method_doc
    def projectReferencesOntoSketch(
        self,
        sketch: str,
        filter: SymbolicConstant = ALL_EDGES,
        upToFeature: Optional[AssemblyFeature] = None,
        edges: tuple = (),
        vertices: tuple = (),
    ):
        """This method projects the specified edges, vertices, and datum points from the assembly
        onto the specified ConstrainedSketch object. The edges, vertices, and datum points
        appear on the sketch as reference geometry.

        Parameters
        ----------
        sketch
            The ConstrainedSketch object on which the edges, vertices, and datum points are
            projected.
        filter
            A SymbolicConstant specifying how to limit the amount of projection. Possible values are
            ALL_EDGES and COPLANAR_EDGES. If **filter** = COPLANAR_EDGES, edges that are coplanar to the
            sketching plane are the only candidates for projection. The default value is ALL_EDGES.
        upToFeature
            A :py:class:`~abaqus.Feature.Feature.Feature` object specifying a marker in the feature-based history of the part.
            Abaqus/CAE projects onto the sketch only the part entities that were created before the
            feature specified by this marker. By default, all part entities are candidates for
            projection.
        edges
            A sequence of candidate edges to be projected onto the sketch. By default, all edges are
            candidates for projection.
        vertices
            A sequence of candidate vertices to be projected onto the sketch. By default, all
            vertices are candidates for projection.
        """
        ...

    @abaqus_method_doc
    def queryCachedStates(self):
        """This method displays the position of geometric states relative to the sequence of
        features in the assembly cache. The output is displayed in the message area.
        """
        ...

    @abaqus_method_doc
    def regenerate(self):
        """This method regenerates the assembly and brings it up to date with the latest values of
        the assembly parameters. When you modify features of an assembly, it may be convenient
        to postpone regeneration until you make all your changes, since regeneration can be time
        consuming. In contrast, when you modify features of a part that is included in the
        assembly, you should use this command to regenerate the assembly. When you regenerate
        the assembly, it will reflect the changes that you made to the part.
        """
        ...

    @abaqus_method_doc
    def regenerationWarnings(self):
        """This method prints any regeneration warnings associated with the features."""
        ...

    @abaqus_method_doc
    def restore(self):
        """This method restores the parameters of all features in the assembly to the value they
        had before a failed regeneration. Use the restore method after a failed regeneration,
        followed by a regenerate command.
        """
        ...

    @abaqus_method_doc
    def resumeAllFeatures(self):
        """This method resumes all the suppressed features in the part or assembly."""
        ...

    @abaqus_method_doc
    def resumeFeatures(self, featureNames: tuple):
        """This method resumes the specified suppressed features in the assembly.

        Parameters
        ----------
        featureNames
            A sequence of Strings specifying the names of features to resume.
        """
        ...

    @abaqus_method_doc
    def resumeLastSetFeatures(self):
        """This method resumes the last set of features to be suppressed in the assembly."""
        ...

    @abaqus_method_doc
    def rotate(
        self, 
        instanceList: Sequence[str],
        axisPoint: Sequence[float], 
        axisDirection: Sequence[float], 
        angle: float,
    ):
        """This method rotates given instances by the specified amount.

        Parameters
        ----------
        instanceList
            A sequence of Strings specifying the names of instances to rotate.
        axisPoint
            A sequence of three Floats specifying the coordinates of a point on the axis.
        axisDirection
            A sequence of three Floats specifying the direction of the axis.
        angle
            A Float specifying the rotation angle in degrees. Use the right-hand rule to determine
            the direction.
        """
        ...

    @abaqus_method_doc
    def translate(self, instanceList: tuple, vector: tuple):
        """This method translates given instances by the specified amount.

        Parameters
        ----------
        instanceList
            A sequence of Strings specifying the names of instances to translate.
        vector
            A sequence of three Floats specifying a translation vector.
        """
        ...

    @abaqus_method_doc
    def saveGeometryCache(self):
        """This method caches the current geometry, which improves regeneration performance."""
        ...

    @abaqus_method_doc
    def setValues(self, regenerateConstraintsTogether: Boolean):
        """This method modifies the behavior associated with the specified assembly.

        Parameters
        ----------
        regenerateConstraintsTogether
            A Boolean specifying whether the positioning constraints in the assembly should be
            regenerated together before regenerating other assembly features. The default value is
            ON.If the assembly has position constraint features and you modify the value of
            **regenerateConstraintsTogether**, Abaqus/CAE will regenerate the assembly features.

        Raises
        ------
        FeatureError: Regeneration failed
            If one or more features in the assembly fails to regenerate
        """
        ...

    @abaqus_method_doc
    def suppressFeatures(self, featureNames: tuple):
        """This method suppresses specified features.

        Parameters
        ----------
        featureNames
            A sequence of Strings specifying the names of features to suppress in the assembly.
        """
        ...

    @abaqus_method_doc
    def unlinkInstances(self, instances: Sequence[PartInstance]):
        """This method converts the specified PartInstance objects from linked child instances to
        regular instances. The parts associated with the selected instances will be converted to
        regular parts as well.

        Parameters
        ----------
        instances
            A sequence of PartInstance objects to be converted to regular part instances.
        """
        ...

    @abaqus_method_doc
    def writeAcisFile(self, fileName: str, version: Optional[float] = None):
        """This method exports the assembly to a named file in ACIS part (SAT) or assembly (ASAT)
        format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write. The file name's extension is
            used to determine whether a part or assembly is written. Use the file extension .asat
            for the assembly format.
        version
            A Float specifying the ACIS version. For example, the Float 12.0 corresponds to ACIS
            Version 12.0. The default value is the current version of ACIS.
        """
        ...

    @abaqus_method_doc
    def writeCADParameters(
        self, paramFile: str, modifiedParams: tuple = (), updatePaths: str = ""
    ):
        """This method writes the parameters that were imported from the CAD system to a parameter
        file.

        Parameters
        ----------
        paramFile
            A String specifying the parameter file name.
        modifiedParams
            A tuple of tuples each containing the part name, the parameter name and the modified
            parameter value. Default is an empty tuple.
        updatePaths
            A Bool specifying whether to update the path of the CAD model file specified in the
            **parameterFile** to the current directory, if the CAD model is present in the current
            directory.
        """
        ...

    @abaqus_method_doc
    def lock(self):
        """This method locks the assembly. Locking the assembly prevents any further changes to the
        assembly that can trigger regeneration of the assembly.
        """
        ...

    @abaqus_method_doc
    def unlock(self):
        """This method unlocks the assembly. Unlocking the assembly allows it to be regenerated
        after any modifications to the assembly.
        """
        ...

    @abaqus_method_doc
    def setMeshNumberingControl(
        self,
        instances: Sequence[PartInstance],
        startNodeLabel: Optional[int] = None,
        startElemLabel: Optional[int] = None,
    ):
        """This method changes the start node and/or element labels on the specified independent
        part instances before or after Abaqus/CAE generates the meshes. For the meshed
        instances, Abaqus/CAE changes the node and/or element labels while preserving the
        original order and incrementation.

        Parameters
        ----------
        instances
            A sequence of PartInstance objects to change the start node and/or element labels.
        startNodeLabel
            A positive Integer specifying the new start node label.
        startElemLabel
            A positive Integer specifying the new start element label.
        """
        ...

    @abaqus_method_doc
    def copyMeshPattern(
        self,
        elements: Sequence[MeshElement] = (),
        faces: Sequence[Face] = (),
        elemFaces: Sequence[MeshFace] = (),
        targetFace: Optional[MeshFace] = None,
        nodes: Sequence[MeshNode] = (),
        coordinates: tuple = (),
    ):
        """This method copies a mesh pattern from a source region consisting of a set of shell
        elements or element faces onto a target face, mapping nodes and elements in a one-one
        correspondence between source and target.

        Parameters
        ----------
        elements
            A sequence of MeshElement objects or a Set object containing elements and specifying the
            source region.
        faces
            A sequence of Face objects that have associated with shell elements or element faces and
            specifying the source region.
        elemFaces
            A sequence of MeshFace objects specifying the source region.
        targetFace
            A :py:class:`~abaqus.Mesh.MeshFace.MeshFace` object specifying the target region. The target face can be of a different
            part instance.
        nodes
            A sequence of MeshNode objects or a Set object containing nodes on the boundary of
            source region which are to be positioned to the boundary of target face.
        coordinates
            A sequence of three-dimensional coordinate tuples specifying the coordinates for each of
            the given nodes. When specified, the number of coordinate tuples must match the number
            of given nodes, and be ordered to correspond to the given nodes in *ascending order*
            according to index. These coordinates are positions of the nodes of a mesh that will be
            the target face corresponding to nodes provided.
        """
        ...

    @abaqus_method_doc
    def smoothNodes(
        self, nodes: Sequence[MeshNode] = ()
    ):
        """This method smooths the given nodes of a native mesh, moving them locally to a more
        optimal location that improves the quality of the mesh

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects or a Set object containing nodes.

            .. versionchanged:: 2020
                The `coordinates` arguments was removed, the `nodes` now replaces it.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .PorousFailureCriteria import PorousFailureCriteria
from .VoidNucleation import VoidNucleation
from .....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class PorousMetalPlasticity:
    """The PorousMetalPlasticity object specifies a porous metal plasticity model.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].porousMetalPlasticity
            import odbMaterial
            session.odbs[name].materials[name].porousMetalPlasticity

        The table data for this object are:

        - :math:`q_1`.
        - :math:`q_2`.
        - :math:`q_3`.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - POROUS METAL PLASTICITY
    """

    #: A :py:class:`~abaqus.Material.Plastic.Metal.Porous.PorousFailureCriteria.PorousFailureCriteria` object.
    porousFailureCriteria: PorousFailureCriteria = PorousFailureCriteria()

    #: A :py:class:`~abaqus.Material.Plastic.Metal.Porous.VoidNucleation.VoidNucleation` object.
    voidNucleation: VoidNucleation = VoidNucleation(((),))

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        relativeDensity: Optional[float] = None,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a PorousMetalPlasticity object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].PorousMetalPlasticity
                session.odbs[name].materials[name].PorousMetalPlasticity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        relativeDensity
            None or a Float specifying the initial relative density of the material, r0. The default
            value is None.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        PorousMetalPlasticity
            A :py:class:`~abaqus.Material.Plastic.Metal.Porous.PorousMetalPlasticity.PorousMetalPlasticity` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the PorousMetalPlasticity object.

        Raises
        ------
        RangeError
        """
        ...



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ...UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class ConstrainedSketchImageOptions:
    """The ConstrainedSketchImageOptions object is used to store values and attributes
    associated with the background image for a particular sketch. The
    ConstrainedSketchImageOptions object has no constructor.

    .. note:: 
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name].imageOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        imageName: str = "",
        showImage: Boolean = OFF,
        origin: Sequence[float] = (),
        xScale: float = 1,
        yScale: float = 1,
        translucency: float = 1,
    ):
        """This method modifies the ConstrainedSketchOptions object.

        Parameters
        ----------
        imageName
            A String specifying the name of the image. A list of valid image names is in the
            **images** repository in the **session** object.
        showImage
            A Boolean specifying whether an image should be displayed in the sketcher background.
            The default value is OFF.
        origin
            A pair of Floats specifying the **X**- and **Y**-offsets in millimeters from the lower-left
            corner of the viewport. The default value is (0, 0).
        xScale
            A Float specifying the scale applied to the image width. The default value is 1.0.When
            **xScale** is negative, the image is mirrored about its y-axis but its position is not
            affected.
        yScale
            A Float specifying the scale applied to the image height. The default value is 1.0.When
            **yScale** is negative, the image is mirrored about its x-axis but its position is not
            affected.
        translucency
            A Float specifying the translucency factor to use when displaying the image. Possible
            values are 0.0 ≤ **translucency** ≤ 1.0 with 0.0 being invisible and 1.0 being opaque.
            The default value is 1.0.

        Raises
        ------
        RangeError

        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .SolidSection import SolidSection


@abaqus_class_doc
class HomogeneousSolidSection(SolidSection):
    """The HomogeneousSolidSection object defines the properties of a solid section.
    The HomogeneousSolidSection object is derived from the SolidSection object.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]

        The corresponding analysis keywords are:

        - SOLID SECTION
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the material.
    material: str

    #: A Float specifying the thickness of the section. Possible values are None or greater
    #: than zero. The default value is 1.0.
    #:
    #: .. versionchanged:: 2018
    #:    The default value is now 1.0 instead of None.
    thickness: Optional[float] = None

    @abaqus_method_doc
    def __init__(
        self, name: str, material: str, thickness: Optional[float] = None
    ) -> None:
        """This method creates a HomogeneousSolidSection object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].HomogeneousSolidSection
                session.odbs[name].HomogeneousSolidSection

        Parameters
        ----------
        name
            A String specifying the repository key.
        material
            A String specifying the name of the material.
        thickness
            A Float specifying the thickness of the section. Possible values are None or greater
            than zero. The default value is 1.0.

            .. versionchanged:: 2018
                The default value is now 1.0 instead of None.

        Returns
        -------
        HomogeneousSolidSection
            A :py:class:`~abaqus.Section.HomogeneousSolidSection.HomogeneousSolidSection` object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, thickness: float = 1) -> None:
        """This method modifies the HomogeneousSolidSection object.

        Parameters
        ----------
        thickness
            A Float specifying the thickness of the section. Possible values are None or greater
            than zero. The default value is 1.0.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing import Optional, Union, Sequence, overload

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Region import Region
from .RegionAssemblyBase import RegionAssemblyBase
from .Set import Set
from .Surface import Surface
from ..BasicGeometry.Cell import Cell
from ..BasicGeometry.Edge import Edge
from ..BasicGeometry.Face import Face
from ..BasicGeometry.ReferencePoint import ReferencePoint
from ..BasicGeometry.Vertex import Vertex
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshNode import MeshNode


@abaqus_class_doc
class RegionAssembly(RegionAssemblyBase):
    """An :py:class:`~abaqus.Assembly.Assembly.Assembly` object is a container for instances of parts. The Assembly object has no
    constructor command. Abaqus creates the **rootAssembly** member when a Model object is
    created.

    .. note:: 
        This object can be accessed by::

            import assembly
            mdb.models[name].rootAssembly
    """

    @abaqus_method_doc
    def Surface(
        self,
        side1Faces: Union[Face, Sequence[Face], None] = None,
        side2Faces: Union[Face, Sequence[Face], None] = None,
        side12Faces: Union[Face, Sequence[Face], None] = None,
        end1Edges: Union[Face, Sequence[Face], None] = None,
        end2Edges: Union[Face, Sequence[Face], None] = None,
        circumEdges: Union[Face, Sequence[Face], None] = None,
        side1Edges: Union[Face, Sequence[Face], None] = None,
        side2Edges: Union[Face, Sequence[Face], None] = None,
        face1Elements: Union[Face, Sequence[Face], None] = None,
        face2Elements: Union[Face, Sequence[Face], None] = None,
        face3Elements: Union[Face, Sequence[Face], None] = None,
        face4Elements: Union[Face, Sequence[Face], None] = None,
        face5Elements: Union[Face, Sequence[Face], None] = None,
        face6Elements: Union[Face, Sequence[Face], None] = None,
        side1Elements: Union[Face, Sequence[Face], None] = None,
        side2Elements: Union[Face, Sequence[Face], None] = None,
        side12Elements: Union[Face, Sequence[Face], None] = None,
        end1Elements: Union[Face, Sequence[Face], None] = None,
        end2Elements: Union[Face, Sequence[Face], None] = None,
        circumElements: Union[Face, Sequence[Face], None] = None,
        name: str = "",
    ) -> Surface:
        """This method creates a surface from a sequence of objects in a model database. The
        surface will apply to the sides specified by the arguments.For example
        surface=mdb.models['Model-1'].parts['Part-1'].Surface(side1Faces=side1Faces,
        name='Surf-1')

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Surface
                mdb.models[name].rootAssembly.Surface

        Parameters
        ----------
        name
            A String specifying the repository key. The default value is an empty string.
        side1Elements
            A sequence of MeshElement objects (surface applies to SIDE1 of element). 
            The default value is None.
        side2Elements
            A sequence of MeshElement objects (surface applies to SIDE2 of element). 
            The default value is None.
        side12Elements
            A sequence of MeshElement objects (surface applies to both SIDE1 and SIDE2 of element). 
            The default value is None.
        end1Elements
            A sequence of MeshElement objects (surface applies to END1 of element). 
            The default value is None.
        end2Elements
            A sequence of MeshElement objects (surface applies to END2 of element). 
            The default value is None.
        circumElements
            A sequence of MeshElement objects (surface applies to circumference of element). 
            The default value is None.
        face1Elements
            A sequence of MeshElement objects (surface applies to FACE1 of element) or MeshFace objects. 
            The default value is None.
        face2Elements
            A sequence of MeshElement objects (surface applies to FACE2 of element) or MeshFace objects. 
            The default value is None.
        face3Elements
            A sequence of MeshElement objects (surface applies to FACE3 of element) or MeshFace objects. 
            The default value is None.
        face4Elements
            A sequence of MeshElement objects (surface applies to FACE4 of element) or MeshFace objects. 
            The default value is None.
        face5Elements
            A sequence of MeshElement objects (surface applies to FACE5 of element) or MeshFace objects. 
            The default value is None.
        face6Elements
            A sequence of MeshElement objects (surface applies to FACE6 of element) or MeshFace objects. 
            The default value is None.
        side1Faces
            A sequence of Face objects (surface applies to SIDE1 of face). The default value is None.
        side2Faces
            A sequence of Face objects (surface applies to SIDE2 of face). The default value is None.
        side12Faces
            A sequence of Face objects (surface applies to both SIDE1 and SIDE2 of face). 
            The default value is None.
        side1Edges
            A sequence of Edge objects (surface applies to SIDE1 of edge). The default value is None.
        side2Edges
            A sequence of Edge objects (surface applies to SIDE2 of edge). The default value is None.
        end1Edges
            A sequence of Edge objects (surface applies to END1 of edge). The default value is None.
        end2Edges
            A sequence of Edge objects (surface applies to END2 of edge). The default value is None.
        circumEdges
            A sequence of Edge objects (surface applies circumferentially to edge). 
            The default value is None.
        kwargs
            The required parameters for different conditions are:
            
            - On three-dimensional solid faces, you can use the following arguments: 
              side1Faces, side2Faces

            - On three-dimensional shell faces, you can use the following arguments: 
              side1Faces, side2Faces, side12Faces

            - On three-dimensional wire edges, you can use the following arguments:
              end1Edges, end2Edges, circumEdges

            - On three-dimensional or two-dimensional or axisymmetric edges, you can use the following arguments:
              side1Edges, side2Edges

            - On two-dimensional or axisymmetric shell elements, you can use the following arguments:
              face1Elements, face2Elements, face3Elements, face4Elements

            - On solid elements, you can use the following arguments:
              face1Elements, face2Elements, face3Elements, face4Elements, face5Elements, face6Elements

            - On three-dimensional shell elements, you can use the following arguments:
              side1Elements, side2Elements, side12Elements

            - On three-dimensional wire elements, you can use the following arguments:
              end1Elements, end2Elements, circumElements

            - On two-dimensional or axisymmetric wire elements, you can use the following arguments:
              side1Elements, side2Elements

        Returns
        -------
        Surface
            A :py:class:`~abaqus.Region.Surface.Surface` object.
        """
        surface = Surface(
            side1Faces,
            side2Faces,
            side12Faces,
            end1Edges,
            end2Edges,
            circumEdges,
            side1Edges,
            side2Edges,
            face1Elements,
            face2Elements,
            face3Elements,
            face4Elements,
            face5Elements,
            face6Elements,
            side1Elements,
            side2Elements,
            side12Elements,
            end1Elements,
            end2Elements,
            circumElements,
            name,
        )
        self.surfaces[name] = surface
        self.allSurfaces[name] = surface
        return surface

    @overload
    @abaqus_method_doc
    def Set(
        self,
        name: str,
        nodes: Optional[Sequence[MeshNode]] = None,
        elements: Optional[Sequence[MeshElement]] = None,
        region: Optional[Region] = None,
        vertices: Optional[Sequence[Vertex]] = None,
        edges: Optional[Sequence[Edge]] = None,
        faces: Optional[Sequence[Face]] = None,
        cells: Optional[Sequence[Cell]] = None,
        xVertices: Optional[Sequence[Vertex]] = None,
        xEdges: Optional[Sequence[Edge]] = None,
        xFaces: Optional[Sequence[Face]] = None,
        referencePoints: Sequence[ReferencePoint] = (),
        skinFaces: tuple = ...,
        skinEdges: tuple = ...,
        stringerEdges: tuple = ...,
    ) -> Set:
        """This method creates a set from a sequence of objects in a model database.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        nodes
            A sequence of MeshNode objects. The default value is None.
        elements
            A sequence of MeshElement objects. The default value is None.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying other objects to be included in the set. The default value is
            None.
        vertices
            A sequence of ConstrainedSketchVertex objects. The default value is None.
        edges
            A sequence of Edge objects. The default value is None.
        faces
            A sequence of Face objects. The default value is None.
        cells
            A sequence of Cell objects. The default value is None.
        xVertices
            A sequence of ConstrainedSketchVertex objects that excludes specific vertices from the set. The default
            value is None.
        xEdges
            A sequence of Edge objects that excludes specific edges from the set. The default value
            is None.
        xFaces
            A sequence of Face objects that excludes specific faces from the set. The default value
            is None.
        referencePoints
            A sequence of ReferencePoint objects. The default value is an empty sequence.
        skinFaces
            A tuple of tuples specifying a skin name and the sequence of faces associated with this
            skin. Valid only for geometric regions on 3D and 2D parts.
        skinEdges
            A tuple of tuples specifying a skin name and the sequence of edges associated with this
            skin. Valid only for geometric regions on Axisymmetric parts.
        stringerEdges
            A tuple of tuples specifying a stringer name and the sequence of edges associated with
            this stringer. Valid only for geometric regions on 3D and 2D parts.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def Set(self, name: str, objectToCopy: Set) -> Set:
        """This method copies a set from an existing set.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the name of the set.
        objectToCopy
            A :py:class:`~abaqus.Region.Set.Set` object to be copied.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    def Set(self, name, *args, **kwargs) -> Set:
        self.sets[name] = aSet = Set(name, *args, **kwargs)
        return aSet



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import Boolean, OFF, ON


@abaqus_class_doc
class LoadCase:
    """The LoadCase object is used to define the loads and constraints comprising a particular
    loading condition and the linear response of a structure subjected to that loading
    condition.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].loadCases[name]
    """

    #: A Boolean specifying whether the load case is suppressed or not. The default value is
    #: OFF.
    suppressed: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        boundaryConditions: tuple = (),
        loads: tuple = (),
        includeActiveBaseStateBC: Boolean = ON,
    ):
        """This method creates a load case in a step.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].steps[name].LoadCase

        Parameters
        ----------
        name
            A String specifying the name of the object.
        boundaryConditions
            A sequence of (String, Float) sequences specifying the name of a BoundaryCondition
            followed by a nonzero Float scaling factor. The default value is an empty sequence.
        loads
            A sequence of (String, Float) sequences specifying the name of a Load followed by a
            nonzero Float specifying a scale factor. The default value is an empty sequence.
        includeActiveBaseStateBC
            A Boolean specifying whether to include all active boundary conditions propagated or
            modified from the base state. The default value is ON.

        Returns
        -------
        case: LoadCase
            A :py:class:`~abaqus.Load.LoadCase.LoadCase` object
        """
        ...

    @abaqus_method_doc
    def resume(self):
        """This method resumes the load case that was previously suppressed."""
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses the load case."""
        ...

    @abaqus_method_doc
    def setValues(
        self,
        boundaryConditions: tuple = (),
        loads: tuple = (),
        includeActiveBaseStateBC: Boolean = ON,
    ):
        """This method modifies the LoadCase object.

        Parameters
        ----------
        boundaryConditions
            A sequence of (String, Float) sequences specifying the name of a BoundaryCondition
            followed by a nonzero Float scaling factor. The default value is an empty sequence.
        loads
            A sequence of (String, Float) sequences specifying the name of a Load followed by a
            nonzero Float specifying a scale factor. The default value is an empty sequence.
        includeActiveBaseStateBC
            A Boolean specifying whether to include all active boundary conditions propagated or
            modified from the base state. The default value is ON.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class DamageStabilizationCohesive:
    """The DamageStabilizationCohesive object specifies the viscosity coefficients for the
    damage model for surface-based cohesive behavior or enriched cohesive behavior.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].ductileDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].fldDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].flsdDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].hashinDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].johnsonCookDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].maxeDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].maxpeDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].maxpsDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].maxsDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].mkDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].msfldDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].quadeDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].quadsDamageInitiation.damageStabilizationCohesive
            mdb.models[name].materials[name].shearDamageInitiation.damageStabilizationCohesive
            import odbMaterial
            session.odbs[name].materials[name].ductileDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].fldDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].flsdDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].hashinDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].johnsonCookDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].maxeDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].maxpeDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].maxpsDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].maxsDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].mkDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].msfldDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].quadeDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].quadsDamageInitiation.damageStabilizationCohesive
            session.odbs[name].materials[name].shearDamageInitiation.damageStabilizationCohesive

        The corresponding analysis keywords are:

        - DAMAGE STABILIZATION
    """

    @abaqus_method_doc
    def __init__(self, cohesiveCoeff: Optional[float] = None):
        """This method creates a DamageStabilizationCohesive object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].ductileDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].fldDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].flsdDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].hashinDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].johnsonCookDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].maxeDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].maxpeDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].maxpsDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].maxsDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].mkDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].msfldDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].quadeDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].quadsDamageInitiation.DamageStabilizationCohesive
                mdb.models[name].materials[name].shearDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].ductileDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].fldDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].flsdDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].hashinDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].johnsonCookDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].maxeDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].maxpeDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].maxpsDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].maxsDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].mkDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].msfldDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].quadeDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].quadsDamageInitiation.DamageStabilizationCohesive
                session.odbs[name].materials[name].shearDamageInitiation.DamageStabilizationCohesive
            
        Parameters
        ----------
        cohesiveCoeff
            None or a Float specifying the viscosity coefficient. The default value is None. 

        Returns
        -------
        DamageStabilizationCohesive
            A :py:class:`~abaqus.Material.ProgressiveDamageFailure.DamageStabilizationCohesive.DamageStabilizationCohesive` object. 

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the DamageStabilizationCohesive object.

        Raises
        ------
        RangeError
        """
        ...



import argparse
import os

epilog = """
Options
=======
The default execution for abaqus command line options is to use `-n` (noGUI mode)
with no additional command line options.

When using abaqus command line options (`-g`, `-n`, `-p`) together with the python 
`script`, and the script is passed after abaqus command line options, you may need 
to specify `--` before the script filename, to prevent `abqpy` from attempting to 
parse it to abaqus. That is not necessary if you pass the filename before abaqus 
command line options.

If you pass N arguments to your python script with the `args` option, beware that the 
abaqus python interpreter will not access them in `sys.argv[1:N]`, but `sys.argv[N:]`, 
because abaqus command already parses some arguments to its python interpreter, depending 
on the chosen options. Please, refer to abaqus documentation for details.

Possible `abaqus cae` command line options:

    database=database-file
    replay=replay-file
    recover=journal-file
    startup=startup-file
    noenvstartup
    noSavedOptions
    noSavedGuiPrefs
    noStartupDialog
    custom=script-file
    guiTester=GUI-script
    guiRecord
    guiNoRecord
    
Possible `abaqus python` command line options:

    sim=sim_file_name
    log=log_file_name

Please, refer to abaqus documentation for details in each of the above options.

"""


def cli():
    parser = argparse.ArgumentParser(
        description="The abqpy command line interface",
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "script", metavar="script", type=str, nargs="?", help="the python script to run"
    )
    parser.add_argument(
        "args", nargs="*", help="arguments that will be passed to the python script",
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-g",
        "--cae-gui",
        dest="gui",
        nargs="*",
        metavar="options",
        help="command line options used to run Abaqus/CAE command with the graphical user interface (GUI mode)",
    )
    group.add_argument(
        "-n",
        "--cae-nogui",
        dest="nogui",
        nargs="*",
        metavar="options",
        help="command line options used to run Abaqus/CAE command without the graphical user interface (noGUI mode)",
    )
    group.add_argument(
        "-p",
        "--python",
        dest="python",
        nargs="*",
        metavar="options",
        help="command line options used to run Abaqus Python command",
    )
    parser.add_argument(
        "--",
        dest="sep",
        action="store_true",
        help="argument to pass the script after abaqus command line options",
    )
    args = parser.parse_args()

    abaqus = os.environ.get("ABAQUS_BAT_PATH", "abaqus")
    proc = "cae"
    mode = f"noGUI={args.script}" if args.script else "noGUI"
    sep = "--" if args.args else ""
    options = args.gui or args.nogui or args.python or ""
    if args.gui is not None:
        mode = f"script={args.script}" if args.script else ""
    elif args.python is not None:
        proc = "python"
        sep = ""
        mode = args.script if args.script else ""

    cmd = f"{abaqus} {proc} {mode} {' '.join(options)} {sep} {' '.join(args.args)}".strip()
    message = f"Running the following abaqus command: {cmd}"
    print("", "-" * len(message), message, "-" * len(message), sep="\n")
    os.system(cmd)


if __name__ == "__main__":
    cli()



from typing import Union
from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .PredefinedField import PredefinedField
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import Boolean, UNIFORM, CONSTANT_RATIO, MAGNITUDE


@abaqus_class_doc
class VoidsRatio(PredefinedField):
    """The VoidsRatio object stores the data for an initial void ratio predefined field.
    The VoidsRatio object is derived from the PredefinedField object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name]

        The corresponding analysis keywords are:

        - INITIAL CONDITIONS
    """

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        voidsRatio1: float = ...,
        voidsRatio2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        ratio2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        ratio2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: Boolean = ...,
    ) -> None:
        """This method creates a PorePressure predefined field object.

        .. note::
            This function can be accessed by::

                mdb.models[name].VoidsRatio

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied. Region
            is ignored if the predefined field has **distributionType** = FROM_FILE.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        voidsRatio1
            The initial void ratio in the first region in your model.
        voidsRatio2
            The void ratio of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial void ratio.
        coord2
            The vertical position of the second location in your model
        ratio2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for void ratio at the
            second elevation or an analytical field to define a spatially varying initial void ratio at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        ratio2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `ratio2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            A VoidsRatio object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        voidsRatio1: float = ...,
        voidsRatio2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        ratio2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        ratio2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: bool = ...,
    ) -> None:
        """This method modifies the VoidsRatio object.

        Parameters
        ----------
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        voidsRatio1
            The initial void ratio in the first region in your model.
        voidsRatio2
            The void ratio of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial void ratio.
        coord2
            The vertical position of the second location in your model
        ratio2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for void ratio at the
            second elevation or an analytical field to define a spatially varying initial void ratio at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        ratio2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `ratio2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            None
        """
        ...
